/*
 * Copyright 2004-2005 Jeremias Maerki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.xml.graphics;

import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.URIResolver;


/**
 * <p>The XGProcessor is the equivalent of the JAXP Transformer. It is only 
 * used for simple processing runs with XSL-FO documents. If you need to do 
 * XSLT, too, you should use the {@link XGProcessorHandler}.
 * </p>
 * <p>If you want to listen to events happening during FO processing you can
 * register listeners here. 
 * </p>
 * <p>This is the basic usage pattern of this interface:</p>
 * <pre>
 * //Set up the XGProcessor 
 * XGProcessorFactory factory = new SomeXGProcessorFactory();
 * XGProcessor processor = factory.newXGProcessor();
 *       
 * //Set up the source and target
 * Source src = new StreamSource(new File("C:/temp/helloworld.fo"));
 * XGResult fopres = new StreamXGResult("application/pdf", new File("C:/temp/out.pdf"));
 *
 * //Start the processing run
 * processor.process(src, fopres);
 * </pre>
 * 
 * @author Jeremias Maerki
 */
public interface XGProcessor {

    void addEventListener(XGEventListener listener);
    void removeEventListener(XGEventListener listener);

    void setURIResolver(URIResolver resolver);
    URIResolver getURIResolver();

    /**
     * Allows the user to set specific parameters on the underlying
     * implementation which affect the transformation.
     *
     * @param name The name of the parameter.
     * @param value The value of the parameter.
     * @throws IllegalArgumentException thrown if the underlying
     * implementation doesn't recognize the parameter.
     */
    /*
    void setParameter(String name, Object value)
        throws IllegalArgumentException;
    */

    /**
     * Allows the user to retrieve specific parameters on the underlying
     * implementation.
     * @param name The name of the parameter.
     * @return value The value of the parameter.
     * @throws IllegalArgumentException thrown if the underlying
     * implementation doesn't recognize the parameter.
     */
    /*
    Object getParameter(String name)
        throws IllegalArgumentException;
    */    
    
    /**
     * Processes an XSL-FO document given by the source parameter into the
     * requested output.
     * @param source A JAXP Source object specifying the source XSL-FO file.
     * @param result A XGResult object defining the requested output document.
     * @throws TransformerException In case the transformation fails.
     */
    void process(Source source, XGResult result) throws XGException;

}
