/*
 * Copyright 2004-2005 Jeremias Maerki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.xml.graphics;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import org.xml.graphics.helpers.FactoryFinder;
import org.xml.graphics.helpers.URLBuilder;
import org.xml.graphics.helpers.XGProcessorFactoryProxy;

/**
 * An XGProcessorFactory can be used to instantiate new {@link XGProcessor} or 
 * {@link XGProcessorHandler} instances.
 * 
 * @author Jeremias Maerki
 */
public abstract class XGProcessorFactory {

    protected XGProcessorFactory() {
    }
    
    /**
     * Creates a new XGProcessor instance.
     * @return an XGProcessor
     * @throws XGException if the XGProcessor cannot be instantiated
     */
    public abstract XGProcessor newXGProcessor() throws XGException;
    
    /**
     * Create a new XGProcessorHandler instance.
     * @return an XGProcessorHandler
     * @throws XGException if the XGProcessor cannot be instantiated
     */
    public abstract XGProcessorHandler newXGProcessorHandler() throws XGException;

    /**
     * <p>Look up the value of a feature.
     * </p>
     * <p>The feature name is any absolute URI.
     * </p>
     * 
     * @param name The feature name, which is an absolute URI.
     * @return The current state of the feature (true or false).
     */
    public abstract boolean getFeature(String name);
    
    /**
     * Allows the user to set specific attributes on the underlying
     * implementation. An attribute in this context is defined to
     * be an option that the implementation provides.
     *
     * @param name The name of the attribute.
     * @param value The value of the attribute.
     * @throws IllegalArgumentException thrown if the underlying
     * implementation doesn't recognize the attribute.
     */
    public abstract void setAttribute(String name, Object value)
        throws IllegalArgumentException;

    /**
     * Allows the user to retrieve specific attributes on the underlying
     * implementation.
     * @param name The name of the attribute.
     * @return value The value of the attribute.
     * @throws IllegalArgumentException thrown if the underlying
     * implementation doesn't recognize the attribute.
     */
    public abstract Object getAttribute(String name)
        throws IllegalArgumentException;    
    
    /**
     * Obtain a new instance of a <code>XGProcessorFactory</code>.
     * This static method creates a new factory instance
     * This method uses the following ordered lookup procedure to determine
     * the <code>XGProcessorFactory</code> implementation class to
     * load:
     * <ul>
     * <li>
     * Use the <code>org.xml.graphics.XGProcessorFactory</code> system
     * property.
     * </li>
     * <li>
     * Use the properties file "lib/jaxg.properties" in the JRE directory.
     * This configuration file is in standard <code>java.util.Properties
     * </code> format and contains the fully qualified name of the
     * implementation class with the key being the system property defined
     * above.
     * </li>
     * <li>
     * Use the Services API (as detailed in the JAR specification), if
     * available, to determine the classname. The Services API will look
     * for a classname in the file
     * <code>META-INF/services/org.xml.graphics.XGProcessorFactory</code>
     * in jars available to the runtime.
     * </li>
     * <li>
     * Platform default <code>XGProcessorFactory</code> instance.
     * </li>
     * </ul>
     *
     * Once an application has obtained a reference to a <code>
     * XGProcessorFactory</code> it can use the factory to configure
     * and obtain XGProcessor instances.
     *
     * @return new XGProcessorFactory instance
     */
    public static XGProcessorFactory newInstance() {
        try {
            return (XGProcessorFactory)FactoryFinder.find(
                    "org.xml.graphics.XGProcessorFactory", null);
        } catch (FactoryFinder.ConfigurationError e) {
            throw new XGProcessorFactoryConfigurationError(e.getException(),
                                                           e.getMessage());
        }
    }
    
    /**
     * <p>Obtain an <code>XGProcessorFactory</code> by specifying an XML file 
     * describing a JAXG deployment package that contains all the necessary 
     * libraries. The {@link XGProcessorFactoryProxy} is used to create a 
     * ClassLoader for this package. Like this you can dynamically load and 
     * use different JAXG processor implementations in the same VM.
     * </p>
     * <p>This is what the package format looks like:</p>
     * <pre>
     * &lt;package main-class="[classname of the factory implementation]"&gt;
     *   &lt;jar&gt;[URL to a JAR file]&lt;/jar&gt;
     *   &lt;jar&gt;...&lt;/jar&gt;
     * &lt;/package&gt;
     * </pre>
     * <p>An example:</p>
     * <pre>
     * &lt;package main-class="org.maerki.jaxg.fop.branch.FopBranchXGProcessorFactory"&gt;
     *   &lt;jar&gt;lib/jaxg-ref-impl.jar&lt;/jar&gt;
     *   &lt;jar&gt;lib/fop.jar&lt;/jar&gt;
     *   &lt;jar&gt;lib/batik.jar&lt;/jar&gt;
     *   &lt;jar&gt;lib/avalon-framework-4.2.0.jar&lt;/jar&gt;
     * &lt;/package&gt;
     * </pre>
     * <p>Relative paths are allowed. The URL of the package file will serve as 
     * base URL. To use an implementation deployed on a webserver you would do this:
     * </p>
     * <pre>
     * String location = "http://localhost/fowebservice/fop-branch/package.xml"
     * XGProcessorFactory factory = XGProcessorFactoryProxy.newInstance(location);
     * XGProcessor fop = factory.newXGProcessor();
     * [..]
     * </pre>
     * 
     * @param packageFile the deployment package file to use
     * @return new XGProcessorFactory instance
     * @throws IOException if an error occurs while loading the package file
     */
    public static XGProcessorFactory newInstance(URL packageFile) throws IOException {
        return new XGProcessorFactoryProxy(packageFile);
    }
    
    /**
     * <p>Obtain an <code>XGProcessorFactory</code> by specifying a 
     * JAXG deployment package.
     * </p> 
     * @see #newInstance(java.net.URL)
     * @param packageFile the deployment package file to use
     * @return new XGProcessorFactory instance
     * @throws FileNotFoundException if an error occurs while loading the package file
     */
    public static XGProcessorFactory newInstance(File packageFile) throws IOException {
        try {
            return newInstance(packageFile.toURL());
        } catch (MalformedURLException mfue){
            throw new FileNotFoundException(mfue.getMessage());
        }
    }
    
    /**
     * <p>Obtain an <code>XGProcessorFactory</code> by specifying a 
     * JAXG deployment package.
     * </p> 
     * @see #newInstance(java.net.URL)
     * @param packageFile the deployment package file to use (URL string)
     * @return new XGProcessorFactory instance
     * @throws FileNotFoundException if an error occurs while loading the package file
     */
    public static XGProcessorFactory newInstance(String packageFile) throws IOException {
        try {
            return newInstance(URLBuilder.toURL(packageFile));
        } catch (MalformedURLException mfue) {
            throw new FileNotFoundException(mfue.getMessage());
        }
    }

}