/*
 * Copyright 2004-2005 Jeremias Maerki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.xml.graphics;

import java.util.Properties;

/**
 * <p>This class represents the output to be generated by an FO processor 
 * implementation.
 * </p>
 * <p>You can set output properties to customize the output. For example,
 * you can set document metadata (title, author etc.) or you can customize
 * PDF security.
 * </p>
 * <p>Here's an example which sets document metadata:
 * </p>
 * <pre>
 * XGResult fopres = new StreamXGResult("application/pdf", "C:/Temp/out.pdf");
 * DocumentMetadata metadata = new DocumentMetadata();
 * metadata.setTitle("My test document");
 * fopres.setOutputProperty(DocumentMetadata.KEY, metadata);
 * </pre> 
 * 
 * @author Jeremias Maerki
 */
public abstract class XGResult {
    
    private Properties props = new Properties();
    
    public void setOutputProperty(String key, Object value) {
        this.props.put(key, value);
    }
    
    public Object getOutputProperty(String key, Class expectedClass) {
        Object obj = this.props.get(key);
        if ((obj != null) && !expectedClass.isInstance(obj)) {
            throw new IllegalStateException(
                    "Error retrieving output property with key '" 
                    + key + "'. Expected a " + expectedClass.getName() 
                    + " but got a " + obj.getClass().getName());
        }
        return obj;
    }
    
    public Object getOutputProperty(String key) {
        return getOutputProperty(key, Object.class);
    }
    
    public void setOutputProperties(Properties props) {
        this.props = new Properties(props);
    }
    
    public Properties getOutputProperties() {
        return new Properties(this.props);
    }

}
