/*
 * Copyright 2004-2005 Jeremias Maerki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.xml.graphics.helpers;

import java.io.IOException;
import java.net.URL;

import org.xml.graphics.XGException;
import org.xml.graphics.XGProcessor;
import org.xml.graphics.XGProcessorFactory;
import org.xml.graphics.XGProcessorFactoryConfigurationError;
import org.xml.graphics.XGProcessorHandler;

/**
 * This class is a proxy to other XGProcessorFactory implementations. It
 * is responsible to set up the ClassLoader for a deployment package.
 * 
 * @author Jeremias Maerki
 * @see XGProcessorFactory#newInstance(java.net.URL)
 */
public class XGProcessorFactoryProxy extends XGProcessorFactory {

    private static final boolean DEBUG = false;
    
    private XGProcessorFactory delegate;
    private ClassLoader oldLoader;
    private ClassLoader loader;
    
    public XGProcessorFactoryProxy(URL packageFile) throws IOException {
        if (DEBUG) System.out.println("Loading XGProcessorFactory from " + packageFile);
        prepare(DeploymentPackage.readFromURL(packageFile));
    }
    
    public XGProcessorFactoryProxy(DeploymentPackage pkg) {
        prepare(pkg);
    }
    
    private void prepare(DeploymentPackage pkg) {
        URL[] urls = pkg.getJARs();
        if (DEBUG) System.out.println("Context class loader: " 
                + Thread.currentThread().getContextClassLoader());
        if (DEBUG) System.out.println("this class loader: " + getClass().getClassLoader());
        final boolean useParent = true;
        oldLoader = (useParent ? Thread.currentThread().getContextClassLoader() : null);

        //Create new ClassLoader
        loader = new java.net.URLClassLoader(urls, oldLoader);
        if (DEBUG) System.out.println("New ClassLoader: " + loader);
        if (DEBUG) System.out.println("Instantiating real XGProcessorFactory");

        //This below is probably not necessary.
        //Classes are loaded by the caller object's classloader
        //Thread.currentThread().setContextClassLoader(loader);
        try {
            try {
                Class clazz = Class.forName(pkg.getMainClass(), true, loader);
                //Class clazz = Class.forName(pkg.getMainClass());
                delegate = (XGProcessorFactory)clazz.newInstance();
            } catch (Exception e) {
                throw new XGProcessorFactoryConfigurationError(e,
                    "Error while instantiating XGProcessorFactory"
                        + " from deployment package: " + e.getMessage());
            }
        } finally {
            //Thread.currentThread().setContextClassLoader(oldLoader);
        }
        if (DEBUG) System.out.println("---done.");
    }


    /**
     * @see org.xml.graphics.XGProcessorFactory#newXGProcessor()
     */
    public XGProcessor newXGProcessor() throws XGException {
        return delegate.newXGProcessor();
    }

    /**
     * @see org.xml.graphics.XGProcessorFactory#newXGProcessorHandler()
     */
    public XGProcessorHandler newXGProcessorHandler() throws XGException {
        return delegate.newXGProcessorHandler();
    }


    /**
     * @see org.xml.graphics.XGProcessorFactory#getFeature(java.lang.String)
     */
    public boolean getFeature(String name) {
        return delegate.getFeature(name);
    }


    /**
     * @see org.xml.graphics.XGProcessorFactory#setAttribute(java.lang.String, java.lang.Object)
     */
    public void setAttribute(String name, Object value) throws IllegalArgumentException {
        delegate.setAttribute(name, value);
    }


    /**
     * @see org.xml.graphics.XGProcessorFactory#getAttribute(java.lang.String)
     */
    public Object getAttribute(String name) throws IllegalArgumentException {
        return delegate.getAttribute(name);
    }

}
