/*
 * Copyright 2005 Jeremias Maerki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.maerki.jaxg.batik;

import java.io.IOException;
import java.io.OutputStream;

import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamSource;

import org.apache.batik.transcoder.Transcoder;
import org.apache.batik.transcoder.TranscoderException;
import org.apache.batik.transcoder.TranscoderInput;
import org.apache.batik.transcoder.TranscoderOutput;
import org.apache.batik.transcoder.XMLAbstractTranscoder;

import org.w3c.dom.Document;

import org.xml.graphics.XGException;
import org.xml.graphics.XGResult;
import org.xml.graphics.helpers.AbstractXGProcessor;
import org.xml.graphics.stream.StreamXGResult;

/**
 * XGProcessor implementation for Apache Batik.
 * 
 * @author Jeremias Maerki
 */
public class BatikXGProcessor extends AbstractXGProcessor {

    private TranscoderInput createTranscoderInput(Source source) {
        TranscoderInput input;
        if (source instanceof StreamSource) {
            StreamSource ss = (StreamSource)source;
            if (ss.getInputStream() != null) {
                input = new TranscoderInput(ss.getInputStream());
            } else if (ss.getReader() != null) {
                input = new TranscoderInput(ss.getReader());
            } else if (ss.getPublicId() != null) {
                input = new TranscoderInput(ss.getPublicId());
            } else if (ss.getSystemId() != null) {
                input = new TranscoderInput(ss.getSystemId());
            } else {
                throw new IllegalArgumentException("Could not create a TranscoderInput instance "
                        + "based on the Source instance given: " + source);
            }
        } else if (source instanceof DOMSource) {
            DOMSource ds = (DOMSource)source;
            if (ds.getNode() instanceof Document) {
                input = new TranscoderInput((Document)ds.getNode());
            } else {
                throw new IllegalArgumentException("Node given by a DOMSource must be an "
                        + "instance of Document. Was: " + ds.getNode().getClass().getName());
            }
        } else if (source instanceof SAXSource) {
            SAXSource saxs = (SAXSource)source;
            if (saxs.getXMLReader() != null) {
                input = new TranscoderInput(saxs.getXMLReader());
            } else {
                //TODO InputSource NYI
                throw new IllegalArgumentException("Only passing an XMLReader is supported "
                        + "for SAXSources, InputSource is NYI.");
            }
        } else {
            throw new IllegalArgumentException("Unsupported Source subclass: " 
                    + source.getClass().getName());
        }
        return input;
    }
    
    private Transcoder createTranscoderForMIMEType(String mime) {
        Transcoder transcoder;
        if ("image/png".equals(mime)) {
            transcoder = new org.apache.batik.transcoder.image.PNGTranscoder();
        } else if ("image/jpeg".equals(mime)) {
            transcoder = new org.apache.batik.transcoder.image.JPEGTranscoder();
        } else if ("image/tiff".equals(mime)) {
            transcoder = new org.apache.batik.transcoder.image.TIFFTranscoder();
        } else if ("application/pdf".equals(mime)) {
            transcoder = new org.apache.fop.svg.PDFTranscoder();
        /*} else if ("application/postscript".equals(mime)) {
            transcoder = new org.apache.fop.render.ps.EPSTranscoder();*/
        } else {
            throw new IllegalArgumentException("Unsupported target MIME type: " + mime);
        }
        return transcoder;
    }
    
    public void process(Source source, XGResult result) throws XGException {
        Transcoder transcoder;
        OutputStream ownOutputStream = null;
        OutputStream out;
        if (result instanceof StreamXGResult) {
            StreamXGResult streamResult = (StreamXGResult)result;
            transcoder = createTranscoderForMIMEType(streamResult.getMIMEType());
            out = streamResult.getOutputStream();
            if (out == null) {
                try {
                    String systemID = streamResult.getSystemID(); 
                    if (systemID.startsWith("file:///"))
                    {
                        if (systemID.substring(8).indexOf(":") > 0) {
                            systemID = systemID.substring(8);
                        } else { 
                            systemID = systemID.substring(7);
                        }
                    }
                    out = new java.io.FileOutputStream(systemID);
                    //Buffering for performance
                    out = new java.io.BufferedOutputStream(out);
                } catch (IOException ioe) {
                    throw new XGException(ioe);
                }
                ownOutputStream = out;
            }
        } else {
            throw new UnsupportedOperationException("No OutputStream available");
        }

        transcoder.addTranscodingHint(XMLAbstractTranscoder.KEY_XML_PARSER_VALIDATING, 
                Boolean.FALSE);
        /*
        transcoder.addTranscodingHint(ImageTranscoder.KEY_PIXEL_UNIT_TO_MILLIMETER, 
                new Float((float)(25.4 / dpi))); 
        transcoder.addTranscodingHint(PDFTranscoder.KEY_STROKE_TEXT, Boolean.TRUE);
        */

        //Setup input
        TranscoderInput input = createTranscoderInput(source);
        //input.setURI(svg.toURL().toString());
        //TODO Complete this base baseURL functionality
        
        //Setup output
        try {
            TranscoderOutput output = new TranscoderOutput(out);
            
            try {
                //Do the transformation
                transcoder.transcode(input, output);
            } catch (TranscoderException te) {
                throw new XGException(
                        "An error occured while transcoding the document", te);
            }
        } finally {
            if (ownOutputStream != null) {
                try {
                    ownOutputStream.close();
                } catch (IOException ioe) {
                    //ignore
                }
            }
        }
        
    }

}
