/*
 * Copyright 2005 Jeremias Maerki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.maerki.jaxg.batik;

import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;

import org.apache.batik.dom.svg.SAXSVGDocumentFactory;
import org.apache.batik.dom.svg.SVGDOMImplementation;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.svg.SVGElement;

import org.xml.graphics.XGProcessor;
import org.xml.graphics.XGProcessorHandler;
import org.xml.graphics.XGResult;

import org.xml.sax.SAXException;

/**
 * FOProcessorHandler implementation for Apache Batik. First it builds a SVG DOM 
 * from the SAX stream and then sends it to the transcoder for processing.
 * 
 * @author Jeremias Maerki
 */
public class BatikXGProcessorHandler extends SAXSVGDocumentFactory implements XGProcessorHandler {

    private static final String SAX_PARSER
            = "org.apache.xerces.parsers.SAXParser";

    private BatikXGProcessor processor;
    private XGResult result;

    /**
     * Main constructor
     */
    public BatikXGProcessorHandler() {
        super(SAX_PARSER);
        this.processor = new BatikXGProcessor();
    }

    public void setResult(XGResult result) {
        if (this.result != null) {
            throw new IllegalStateException("Result already set");
        }
        this.result = result;
    }

    public XGProcessor getXGProcessor() {
        return this.processor;
    }
    
    protected void notifyDOMBuilt(Document doc) throws TransformerException {
        Source src = new DOMSource(doc);
        Element rootEl = doc.getDocumentElement();
        if (rootEl instanceof SVGElement) {
            this.processor.process(src, result);
        } else {
            if (rootEl.getNamespaceURI() != null) {
                throw new TransformerException("XML passed in must be SVG, found: " 
                        + rootEl.getNamespaceURI());
            } else {
                throw new TransformerException("XML passed in must be SVG, found: " 
                        + rootEl.getNodeName());
            }
        }
    }

    /* ----- SAX ContentHandler interface ----- */

    /** @see org.xml.sax.ContentHandler#startDocument() */
    public void startDocument() throws SAXException {
        try {
            // Create SVG Document
            String namespaceURI = SVGDOMImplementation.SVG_NAMESPACE_URI;
            super.document = implementation.createDocument(namespaceURI, "svg", null);
            super.startDocument();
            // Add svg, and SVG_NAMESPACE to SAXDocumentFactory namespace handling.
            // This ties 'svg' prefix used above to the svg namespace uri.
            namespaces.put("svg", SVGDOMImplementation.SVG_NAMESPACE_URI);
        } catch (SAXException se) {
            throw se;
        } catch (Exception ex){
            throw new SAXException("Exception in startDocument()", ex);
        }
    }
    
    /** @see org.xml.sax.ContentHandler#endDocument() */
    public void endDocument() throws SAXException {
        super.endDocument();
        try {
            notifyDOMBuilt(super.document);
        } catch (TransformerException te) {
            throw new SAXException("Exception while transcoding the SVG DOM", te);
        }
    }
}
