/*
 * Copyright 2004-2005 Jeremias Maerki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.maerki.jaxg.fop.branch;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.avalon.framework.logger.Logger;
import org.apache.fop.apps.FOPException;
import org.apache.fop.apps.Options;
import org.apache.fop.configuration.Configuration;
import org.apache.fop.messaging.MessageHandler;
import org.xml.graphics.XGProcessor;
import org.xml.graphics.XGProcessorFactory;
import org.xml.graphics.XGProcessorHandler;
import org.xml.graphics.helpers.URLBuilder;
import org.xml.graphics.stream.StreamXGResult;
import org.xml.sax.InputSource;

/**
 * This is the XGProcessorFactory implementation for the FOP maintenance branch
 * (FOP 0.20.5).
 * 
 * @author Jeremias Maerki
 */
public class FopBranchXGProcessorFactory extends XGProcessorFactory {

    private Logger logger;
    
    /**
     * @see org.xml.graphics.XGProcessorFactory#newXGProcessor()
     */
    public XGProcessor newXGProcessor() {
        FopBranchXGProcessor processor = new FopBranchXGProcessor();
        processor.setLogger(this.logger);
        return processor;
    }

    /**
     * @see org.xml.graphics.XGProcessorFactory#newXGProcessorHandler()
     */
    public XGProcessorHandler newXGProcessorHandler() {
        FopBranchXGProcessorHandler handler = new FopBranchXGProcessorHandler();
        ((FopBranchXGProcessor)handler.getXGProcessor()).setLogger(this.logger);
        return handler;
    }

    /**
     * @see org.xml.graphics.XGProcessorFactory#getFeature(java.lang.String)
     */
    public boolean getFeature(String name) {
        return StreamXGResult.FEATURE.equals(name);
    }

    /**
     * @see org.xml.graphics.XGProcessorFactory#setAttribute(java.lang.String, java.lang.Object)
     */
    public void setAttribute(String name, Object value) throws IllegalArgumentException {
        if (Constants.ATTR_USERCONFIG.equals(name)) {
            try {
                if (value instanceof File) {
                    new Options((File)value);
                } else if (value instanceof String) {
                    try {
                        File f = URLBuilder.toFile((String)value);
                        if (f != null) {
                            new Options(f);
                        } else {
                            URL url = URLBuilder.toURL((String)value);
                            InputStream in = url.openStream();
                            try {
                                new Options(in);
                            } finally {
                                in.close();
                            }
                        }
                    } catch (MalformedURLException e) {
                        throw new IllegalArgumentException(
                                "Cannot load configuration using: " 
                                + value + " (" + e.getMessage() + ")");
                    } catch (IOException e) {
                        throw new IllegalArgumentException(
                                "I/O error while loading configuration from: " 
                                + value + " (" + e.getMessage() + ")");
                    }
                } else if (value instanceof InputStream) {
                    new Options((InputStream)value);
                } else if (value instanceof InputSource) {
                    new Options((InputSource)value);
                } else {
                    throw new IllegalArgumentException("Unsupported value. "
                            + "Expected a java.io.File, a filename, an "
                            + "InputStream or an InputSource.");
                }
            } catch (FOPException fope) {
                throw new IllegalArgumentException(fope.getMessage());
            }
        } else if (Constants.ATTR_BASE_URL.equals(name)) {
            Configuration.put("baseDir", value.toString());
        } else if (Constants.ATTR_AVALON_LOGGER.equals(name)) {
            this.logger = (Logger)value;
            MessageHandler.setScreenLogger(this.logger);
        } else {
            throw new IllegalArgumentException("Unsupported attribute: " + name
                    + ".");
        }
    }

    /**
     * @see org.xml.graphics.XGProcessorFactory#getAttribute(java.lang.String)
     */
    public Object getAttribute(String name) throws IllegalArgumentException {
        throw new IllegalArgumentException("getAttribute() NYI");
    }

}
