/*
 * Copyright 2004-2005 Jeremias Maerki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.maerki.jaxg.fop.branch;

import java.io.IOException;
import java.io.OutputStream;

import javax.print.DocFlavor;
import javax.print.SimpleDoc;

import org.apache.fop.apps.Driver;
import org.apache.fop.fo.FOTreeBuilder;
import org.apache.fop.render.Renderer;
import org.apache.fop.render.awt.AWTRenderer;
import org.xml.graphics.XGResult;
import org.xml.graphics.XGProcessor;
import org.xml.graphics.XGProcessorHandler;
import org.xml.graphics.print.DocXGResult;
import org.xml.graphics.stream.StreamXGResult;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * XGProcessorHandler implementation for Apache FOP.
 * 
 * @author Jeremias Maerki
 */
public class FopBranchXGProcessorHandler implements XGProcessorHandler {

    private FopBranchXGProcessor processor;
    
    private XGResult result;
    private Driver driver;
    private FOTreeBuilder treeBuilder;
    
    private OutputStream ownOutputStream;
    private Locator locator;
    
    public FopBranchXGProcessorHandler() {
        this.processor = new FopBranchXGProcessor();
    }
    
    protected FopBranchXGProcessorHandler(FopBranchXGProcessor processor) {
        this.processor = processor;
    }
    
    public void setResult(XGResult result) {
        if (this.result != null) {
            throw new IllegalStateException("Result already set");
        }
        this.result = result;
    }
    
    public XGProcessor getXGProcessor() {
        return this.processor;
    }

    private Renderer getRenderer() {
        return FopBranchXGProcessor.createRenderer(result);
    }
    
    protected void prepareTreeBuilder() throws SAXException {
        if (treeBuilder != null) return;
        this.driver = new Driver();
        if (this.processor.getLogger() != null) {
            driver.setLogger(this.processor.getLogger());
        }
        OutputStream out = null;
        if (result instanceof StreamXGResult) {
            StreamXGResult sres = (StreamXGResult)result;
            out = sres.getOutputStream();
            if (out == null) {
                try {
                    String systemID = sres.getSystemID(); 
                    if (systemID.startsWith("file:///"))
                    {
                        if (systemID.substring(8).indexOf(":") > 0) {
                            systemID = systemID.substring(8);
                        } else { 
                            systemID = systemID.substring(7);
                        }
                    }
                    out = new java.io.FileOutputStream(systemID);
                    //Buffering for performance
                    out = new java.io.BufferedOutputStream(out);
                } catch (IOException ioe) {
                    throw new SAXException(ioe);
                }
                ownOutputStream = out;
            }
        } else if (result instanceof DocXGResult) {
            //nop
        } else {
            throw new UnsupportedOperationException("No OutputStream available");
        }
        if (out != null) {
            driver.setOutputStream(out);
        }
        driver.setRenderer(getRenderer());
        
        //We know the the ContentHandler returned by Driver is an FOTreeBuilder
        treeBuilder = (FOTreeBuilder)driver.getContentHandler();
        if (this.locator != null) {
            treeBuilder.setDocumentLocator(this.locator);
            this.locator = null;
        }
    }
    
    /* ----- SAX ContentHandler interface ----- */
    
    /** @see org.xml.sax.ContentHandler#setDocumentLocator(org.xml.sax.Locator) */
    public void setDocumentLocator(Locator locator) {
        if (this.treeBuilder != null) {
            treeBuilder.setDocumentLocator(locator);
        } else {
            //setDocumentLocator may be called before startDocument()
            this.locator = locator;
        }
    }

    /** @see org.xml.sax.ContentHandler#startDocument() */
    public void startDocument() throws SAXException {
        prepareTreeBuilder();
        treeBuilder.startDocument();
    }

    /** @see org.xml.sax.ContentHandler#endDocument() */
    public void endDocument() throws SAXException {
        treeBuilder.endDocument();
        if (ownOutputStream != null) {
            try {
                ownOutputStream.close();
            } catch (IOException ioe) {
                //ignore
            }
            ownOutputStream = null;
        } else if (result instanceof DocXGResult) {
            DocXGResult docResult = (DocXGResult)result; 
            DocFlavor flavor = DocFlavor.SERVICE_FORMATTED.PAGEABLE;
            SimpleDoc doc = new SimpleDoc((AWTRenderer)driver.getRenderer(), 
                    flavor, docResult.getDocAttributes());
            docResult.setDoc(doc);
        }
    }

    /** @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String, java.lang.String)*/
    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        treeBuilder.startPrefixMapping(prefix, uri);
    }

    /** @see org.xml.sax.ContentHandler#endPrefixMapping(java.lang.String) */
    public void endPrefixMapping(String prefix) throws SAXException {
        treeBuilder.endPrefixMapping(prefix);
    }

    /** @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes) */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
        treeBuilder.startElement(namespaceURI, localName, qName, atts);
    }

    /** @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String) */
    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
        treeBuilder.endElement(namespaceURI, localName, qName);
    }

    /** @see org.xml.sax.ContentHandler#characters(char[], int, int) */
    public void characters(char[] ch, int start, int length) throws SAXException {
        treeBuilder.characters(ch, start, length);
    }

    /** @see org.xml.sax.ContentHandler#ignorableWhitespace(char[], int, int) */
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
        treeBuilder.ignorableWhitespace(ch, start, length);
    }

    /** @see org.xml.sax.ContentHandler#processingInstruction(java.lang.String, java.lang.String) */
    public void processingInstruction(String target, String data) throws SAXException {
        treeBuilder.processingInstruction(target, data);
    }

    /** @see org.xml.sax.ContentHandler#skippedEntity(java.lang.String) */
    public void skippedEntity(String name) throws SAXException {
        treeBuilder.skippedEntity(name);
    }

}
