/*
 * Copyright 2004-2005 Jeremias Maerki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.maerki.jaxg.fop.head;

import java.io.IOException;
import java.io.OutputStream;
import java.net.URL;

import javax.print.DocFlavor;
import javax.print.SimpleDoc;

import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.container.ContainerUtil;
import org.apache.fop.apps.FOUserAgent;
import org.apache.fop.apps.Fop;
//import org.apache.fop.fo.Constants;
import org.apache.fop.fo.FOEventHandler;
import org.apache.fop.fo.FOTreeBuilder;
import org.apache.fop.render.Renderer;
import org.apache.fop.render.print.PrintRenderer;
import org.xml.graphics.DocumentMetadata;
import org.xml.graphics.XGProcessor;
import org.xml.graphics.XGProcessorHandler;
import org.xml.graphics.XGResult;
import org.xml.graphics.print.DocXGResult;
import org.xml.graphics.stream.StreamXGResult;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * XGProcessorHandler implementation for Apache FOP.
 * 
 * @author Jeremias Maerki
 */
public class FopXGProcessorHandler implements XGProcessorHandler {

    private FopXGProcessor processor;
    
    private XGResult result;
    private Renderer renderer;
    private FOTreeBuilder treeBuilder;
    
    private OutputStream ownOutputStream;
    private Locator locator;
    
    public FopXGProcessorHandler() {
        this.processor = new FopXGProcessor();
    }
    
    protected FopXGProcessorHandler(FopXGProcessor processor) {
        this.processor = processor;
    }
    
    public void setResult(XGResult result) {
        if (this.result != null) {
            throw new IllegalStateException("Result already set");
        }
        this.result = result;
    }
    
    public XGProcessor getXGProcessor() {
        return this.processor;
    }

    private Object getRenderer(FOUserAgent ua, OutputStream out) {
        return FopXGProcessor.createRenderer(result, ua, out);
    }
    
    protected void prepareTreeBuilder() throws SAXException {
        if (treeBuilder != null) return;
        OutputStream out = null;
        if (result instanceof StreamXGResult) {
            StreamXGResult sres = (StreamXGResult)result;
            out = sres.getOutputStream();
            if (out == null) {
                try {
                    String systemID = sres.getSystemID(); 
                    if (systemID.startsWith("file:///"))
                    {
                        if (systemID.substring(8).indexOf(":") > 0) {
                            systemID = systemID.substring(8);
                        } else { 
                            systemID = systemID.substring(7);
                        }
                    }
                    out = new java.io.FileOutputStream(systemID);
                    //Buffering for performance
                    out = new java.io.BufferedOutputStream(out);
                } catch (IOException ioe) {
                    throw new SAXException(ioe);
                }
                ownOutputStream = out;
            }
        } else if (result instanceof DocXGResult) {
            //nop
        } else {
            throw new UnsupportedOperationException("No OutputStream available");
        }
        FOUserAgent ua = new FOUserAgent();
        if (processor.getUserConfig() != null) {
            ua.setUserConfig(processor.getUserConfig());
        }
        
        Object robj = getRenderer(ua, out);
        if (robj instanceof Renderer) {
            renderer = (Renderer)robj;
            renderer.setUserAgent(ua);
            ua.setRendererOverride(renderer);
            Configuration cfg = (Configuration)result.getOutputProperty(
                    Constants.OUT_PROP_AVALON_CONFIG);
            if (cfg != null) {
                try {
                    ContainerUtil.configure(renderer, cfg);
                } catch (ConfigurationException ce) {
                    throw new SAXException(ce);
                }
            }
        } else {
            FOEventHandler handler = (FOEventHandler)robj;
            ua.setFOEventHandlerOverride(handler);
        }
        
        //Document metadata
        DocumentMetadata meta = (DocumentMetadata)result.getOutputProperty(
                DocumentMetadata.KEY, DocumentMetadata.class);
        if (meta != null) {
            ua.setCreator(meta.getCreator());
            ua.setTitle(meta.getTitle());
            ua.setAuthor(meta.getAuthor());
            ua.setKeywords(meta.getKeywords());
        }
        
        ua.setStrictValidation(processor.isStrictValidation());
        
        //Base URL
        Object baseURL = result.getOutputProperty(Constants.ATTR_BASE_URL);
        if (baseURL == null) {
            baseURL = processor.getBaseURL();
        }
        if (baseURL != null) {
            if (baseURL instanceof URL) {
                ua.setBaseURL(baseURL.toString());
            } else if (baseURL instanceof String) {
                ua.setBaseURL((String)baseURL);
            } else {
                throw new IllegalArgumentException(
                        "Unsupported class for baseURL: " 
                            + baseURL.getClass().getName());
            }
            
        }

        //Resolution
        Integer dpi = (Integer)result.getOutputProperty(
                Constants.ATTR_RESOLUTION, Integer.class);
        if (dpi == null) {
            dpi = processor.getResolution();
        }
        if (dpi != null) {
            ua.setResolution(dpi.intValue());
        }

        //TODO The RENDER_PDF here is not very beautiful
        Fop fop = new Fop(org.apache.fop.fo.Constants.RENDER_PDF, ua);
        fop.setOutputStream(out);
        
        //We know the ContentHandler returned by Driver is an FOTreeBuilder
        treeBuilder = (FOTreeBuilder)fop.getDefaultHandler();
        if (this.locator != null) {
            treeBuilder.setDocumentLocator(this.locator);
            this.locator = null;
        }
    }
    
    /* ----- SAX ContentHandler interface ----- */
    
    /** @see org.xml.sax.ContentHandler#setDocumentLocator(org.xml.sax.Locator) */
    public void setDocumentLocator(Locator locator) {
        if (this.treeBuilder != null) {
            treeBuilder.setDocumentLocator(locator);
        } else {
            //setDocumentLocator may be called before startDocument()
            this.locator = locator;
        }
    }

    /** @see org.xml.sax.ContentHandler#startDocument() */
    public void startDocument() throws SAXException {
        prepareTreeBuilder();
        treeBuilder.startDocument();
    }

    /** @see org.xml.sax.ContentHandler#endDocument() */
    public void endDocument() throws SAXException {
        treeBuilder.endDocument();
        if (ownOutputStream != null) {
            try {
                ownOutputStream.close();
            } catch (IOException ioe) {
                //ignore
            }
            ownOutputStream = null;
        } else if (result instanceof DocXGResult) {
            DocXGResult docResult = (DocXGResult)result; 
            DocFlavor flavor = DocFlavor.SERVICE_FORMATTED.PAGEABLE;
            SimpleDoc doc = new SimpleDoc((PrintRenderer)renderer, 
                    flavor, docResult.getDocAttributes());
            docResult.setDoc(doc);
        }
    }

    /** @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String, java.lang.String)*/
    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        treeBuilder.startPrefixMapping(prefix, uri);
    }

    /** @see org.xml.sax.ContentHandler#endPrefixMapping(java.lang.String) */
    public void endPrefixMapping(String prefix) throws SAXException {
        treeBuilder.endPrefixMapping(prefix);
    }

    /** @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes) */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
        treeBuilder.startElement(namespaceURI, localName, qName, atts);
    }

    /** @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String) */
    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
        treeBuilder.endElement(namespaceURI, localName, qName);
    }

    /** @see org.xml.sax.ContentHandler#characters(char[], int, int) */
    public void characters(char[] ch, int start, int length) throws SAXException {
        treeBuilder.characters(ch, start, length);
    }

    /** @see org.xml.sax.ContentHandler#ignorableWhitespace(char[], int, int) */
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
        treeBuilder.ignorableWhitespace(ch, start, length);
    }

    /** @see org.xml.sax.ContentHandler#processingInstruction(java.lang.String, java.lang.String) */
    public void processingInstruction(String target, String data) throws SAXException {
        treeBuilder.processingInstruction(target, data);
    }

    /** @see org.xml.sax.ContentHandler#skippedEntity(java.lang.String) */
    public void skippedEntity(String name) throws SAXException {
        treeBuilder.skippedEntity(name);
    }

}
