/*
 * Copyright 2004-2005 Jeremias Maerki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.maerki.jaxg.foray;

import java.io.File;

import org.apache.avalon.framework.logger.Logger;
import org.apache.avalon.framework.logger.NullLogger;

import org.foray.app.Options;
import org.foray.app.SessionConfig;
import org.foray.area.LayoutStrategy;
import org.foray.common.FOrayException;
import org.foray.output.OutputConfig;
import org.foray.pioneer.PioneerLS;

import org.xml.graphics.XGException;
import org.xml.graphics.XGProcessor;
import org.xml.graphics.XGProcessorFactory;
import org.xml.graphics.XGProcessorHandler;
import org.xml.graphics.stream.StreamXGResult;

import org.xml.sax.InputSource;

/**
 * <p>This is the XGProcessorFactory implementation for FOray (an Apache FOP fork)
 * </p>
 * <P>For this implementation make sure to set the "avalon-logger" value as the first parameter
 * to the factory if you want to use your own logger, otherwise no logging is performed.
 * </p>
 * 
 * @author Jeremias Maerki
 */
public class FOrayXGProcessorFactory extends XGProcessorFactory {

    public static final String ATTR_USERCONFIG = "userconfig";
    public static final String ATTR_BASE_URL = "baseURL";
    public static final String ATTR_AVALON_LOGGER = "avalon-logger";
    
    private Logger logger;
    private SessionConfig sessionConfig;
    private OutputConfig outputConfig;
    private Options options;
    private LayoutStrategy layoutStrategy = new PioneerLS();
    
    private void checkMandatoryAttributes() throws XGException {
        if (this.logger == null) {
            //throw new FOPException("Mandatory attribute '" + ATTR_AVALON_LOGGER + "' not set.");
        }
    }
    
    private void prepareOptions() throws XGException {
        if (this.options == null) {
            if (this.logger == null) {
                this.logger = new NullLogger();
                /*
                throw new IllegalArgumentException("Set '" + ATTR_AVALON_LOGGER 
                        + "' (the Avalon Logger instance) before setting the options file.");
                */
            }
            this.sessionConfig = new SessionConfig(this.logger);
            this.outputConfig = new OutputConfig(this.logger);
            try {
                this.options = new Options(this.logger, this.sessionConfig, this.outputConfig);
            } catch (FOrayException fe) {
                throw new XGException(fe);
            }
        }
    }
    
    private void configure(FOrayXGProcessor processor) throws XGException {
        prepareOptions();
        processor.setLogger(this.logger);
        processor.setOptions(this.options);
        processor.setOutputConfig(this.outputConfig);
        processor.setLayoutStrategy(this.layoutStrategy);
    }
    
    public XGProcessor newXGProcessor() throws XGException {
        checkMandatoryAttributes();
        FOrayXGProcessor processor = new FOrayXGProcessor();
        configure(processor);
        return processor;
    }

    public XGProcessorHandler newXGProcessorHandler() throws XGException {
        checkMandatoryAttributes();
        FOrayXGProcessorHandler handler = new FOrayXGProcessorHandler();
        configure((FOrayXGProcessor)handler.getXGProcessor());
        return handler;
    }

    public boolean getFeature(String name) {
        return StreamXGResult.FEATURE.equals(name);
    }

    public void setAttribute(String name, Object value) throws IllegalArgumentException {
        if (ATTR_USERCONFIG.equals(name)) {
            try {
                prepareOptions();
            } catch (XGException e) {
                throw new IllegalArgumentException(e.getMessage());
            }
            if (value instanceof File) {
                this.options.loadUserconfiguration((File)value);
            } else if (value instanceof String) {
                File f = new File((String)value);
                this.options.loadUserconfiguration(f);
            /*
            } else if (value instanceof InputStream) {
                this.options.loadUserconfiguration((InputStream)value);
            */
            } else if (value instanceof InputSource) {
                this.options.loadUserconfiguration((InputSource)value);
            } else {
                throw new IllegalArgumentException("Unsupported value. "
                        + "Expected a java.io.File, a filename, an "
                        + "InputStream or an InputSource.");
            }
        } else if (ATTR_BASE_URL.equals(name)) {
            //Configuration.put("baseDir", value);
        } else if (ATTR_AVALON_LOGGER.equals(name)) {
            if (this.logger != null) {
                throw new IllegalStateException("The parameter '" + ATTR_AVALON_LOGGER + "' must"
                        + " be set as the first parameter if you want to have logging.");
            }
            this.logger = (Logger)value;
            //MessageHandler.setScreenLogger(this.logger);
        } else {
            throw new IllegalArgumentException("Unsupported attribute: " + name
                    + ". Expected: " + ATTR_USERCONFIG);
        }
    }

    /**
     * @see org.xml.fo.FOProcessorFactory#getAttribute(java.lang.String)
     */
    public Object getAttribute(String name) throws IllegalArgumentException {
        throw new IllegalArgumentException("getAttribute() NYI");
    }

}
