/*
 * Copyright 2004-2005 Jeremias Maerki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.maerki.jaxg.jfor;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;

import org.jfor.jfor.converter.Converter;
import org.xml.graphics.XGProcessor;
import org.xml.graphics.XGProcessorHandler;
import org.xml.graphics.XGResult;
import org.xml.graphics.stream.StreamXGResult;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * XGProcessorHandler implementation for JFOR.
 * 
 * @author Jeremias Maerki
 */
public class JforXGProcessorHandler implements XGProcessorHandler {

    private JforXGProcessor processor;
    
    private XGResult result;
    private Converter converter;
    
    private OutputStream ownOutputStream;
    private Locator locator;
    
    public JforXGProcessorHandler() {
        this.processor = new JforXGProcessor();
    }
    
    protected JforXGProcessorHandler(JforXGProcessor processor) {
        this.processor = processor;
    }
    
    public void setResult(XGResult result) {
        if (this.result != null) {
            throw new IllegalStateException("Result already set");
        }
        this.result = result;
    }
    
    public XGProcessor getXGProcessor() {
        return this.processor;
    }

    protected void prepareConverter() throws SAXException {
        if (converter != null) return;
        OutputStream out;
        if (result instanceof StreamXGResult) {
            StreamXGResult sres = (StreamXGResult)result;
            out = sres.getOutputStream();
            if (out == null) {
                try {
                    String systemID = sres.getSystemID(); 
                    if (systemID.startsWith("file:///"))
                    {
                        if (systemID.substring(8).indexOf(":") > 0) {
                            systemID = systemID.substring(8);
                        } else { 
                            systemID = systemID.substring(7);
                        }
                    }
                    out = new java.io.FileOutputStream(systemID);
                    //Buffering for performance
                    out = new java.io.BufferedOutputStream(out);
                } catch (IOException ioe) {
                    throw new SAXException(ioe);
                }
                ownOutputStream = out;
            }
            if (!("application/rtf".equals(sres.getMIMEType())
                    || "text/rtf".equals(sres.getMIMEType())
                    || "text/richtext".equals(sres.getMIMEType()))) {
                throw new SAXException("Unsupported output format: " 
                        + sres.getMIMEType() + ". Only RTF is supported "
                        + "(application/rtf, text/rtf, text/richtext)");
            }
        } else {
            throw new UnsupportedOperationException("No OutputStream available");
        }

        Writer writer = new java.io.OutputStreamWriter(out);
        try {
            converter = new Converter(writer, Converter.createConverterOption());
        } catch (Exception e) {
            throw new SAXException(e);
        }
        
        if (this.locator != null) {
            converter.setDocumentLocator(this.locator);
            this.locator = null;
        }
    }
    
    /* ----- SAX ContentHandler interface ----- */
    
    /** @see org.xml.sax.ContentHandler#setDocumentLocator(org.xml.sax.Locator) */
    public void setDocumentLocator(Locator locator) {
        if (this.converter != null) {
            converter.setDocumentLocator(locator);
        } else {
            //setDocumentLocator may be called before startDocument()
            this.locator = locator;
        }
    }

    /** @see org.xml.sax.ContentHandler#startDocument() */
    public void startDocument() throws SAXException {
        prepareConverter();
        converter.startDocument();
    }

    /** @see org.xml.sax.ContentHandler#endDocument() */
    public void endDocument() throws SAXException {
        converter.endDocument();
        if (ownOutputStream != null) {
            try {
                ownOutputStream.close();
            } catch (IOException ioe) {
                //ignore
            }
            ownOutputStream = null;
        }
    }

    /** @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String, java.lang.String)*/
    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        converter.startPrefixMapping(prefix, uri);
    }

    /** @see org.xml.sax.ContentHandler#endPrefixMapping(java.lang.String) */
    public void endPrefixMapping(String prefix) throws SAXException {
        converter.endPrefixMapping(prefix);
    }

    /** @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes) */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
        converter.startElement(namespaceURI, localName, qName, atts);
    }

    /** @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String) */
    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
        converter.endElement(namespaceURI, localName, qName);
    }

    /** @see org.xml.sax.ContentHandler#characters(char[], int, int) */
    public void characters(char[] ch, int start, int length) throws SAXException {
        converter.characters(ch, start, length);
    }

    /** @see org.xml.sax.ContentHandler#ignorableWhitespace(char[], int, int) */
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
        converter.ignorableWhitespace(ch, start, length);
    }

    /** @see org.xml.sax.ContentHandler#processingInstruction(java.lang.String, java.lang.String) */
    public void processingInstruction(String target, String data) throws SAXException {
        converter.processingInstruction(target, data);
    }

    /** @see org.xml.sax.ContentHandler#skippedEntity(java.lang.String) */
    public void skippedEntity(String name) throws SAXException {
        converter.skippedEntity(name);
    }

}
